@@ -1,17 +1,19 @@
 package App::sslmaker;
 use strict;
 use warnings;
-use Carp 'confess';
+
+use Carp qw(confess);
 use Path::Tiny;
 use File::umask;
-use constant DEBUG => $ENV{SSLMAKER_DEBUG} ? 1 : 0;
-use constant DEFAULT_BITS => 4096;
-use constant DEFAULT_DAYS => 365;
 
-our $VERSION = '0.13';
+use constant DEBUG        => $ENV{SSLMAKER_DEBUG} ? 1 : 0;
+use constant DEFAULT_BITS => $ENV{SSLMAKER_BITS} || 4096;
+use constant DEFAULT_DAYS => $ENV{SSLMAKER_DAYS} || 365;
+
+our $VERSION = '0.14';
 our $OPENSSL = $ENV{SSLMAKER_OPENSSL} || 'openssl';
 
-my @CONFIG_TEMPLATE_KEYS = qw( bits cert crl_days days home key );
+my @CONFIG_TEMPLATE_KEYS = qw(bits cert crl_days days home key);
 
 # heavily inspired by Mojo::Loader::_all()
 my %DATA = do {
@@ -20,15 +22,15 @@ my %DATA = do {
   $data =~ s/^.*\n__DATA__\r?\n/\n/s;
   $data =~ s/\n__END__\r?\n.*$/\n/s;
   $data = [split /^@@\s*(.+?)\s*\r?\n/m, $data];
-  shift @$data; # first element is empty string
+  shift @$data;    # first element is empty string
   @$data;
 };
 
 # need to be defined up front
 sub openssl {
-  my $cb = ref $_[-1] eq 'CODE' ? pop : sub { warn $_[1] if length $_[1] and DEBUG == 2 };
-  my $self = ref $_[0] ? shift : __PACKAGE__;
-  my $buf = '';
+  my $cb   = ref $_[-1] eq 'CODE' ? pop   : sub { warn $_[1] if length $_[1] and DEBUG == 2 };
+  my $self = ref $_[0]            ? shift : __PACKAGE__;
+  my $buf  = '';
 
   use IPC::Open3;
   use Symbol;
@@ -39,7 +41,7 @@ sub openssl {
   while (1) {
     my $l = sysread $OUT, my $read, 8096;
     confess "$OPENSSL: $!" unless defined $l;
-    last unless $l;
+    last                   unless $l;
     $buf .= $read;
   }
 
@@ -48,24 +50,19 @@ sub openssl {
   $self->$cb($buf);
 }
 
-sub subject {
-  return do { $_[0]->{subject} = $_[1]; $_[0] } if @_ == 2;
-  $_[0]->{subject} //= '/C=NO/ST=Oslo/L=Oslo/O=Example/OU=Prime/CN=example.com/emailAddress=admin@example.com';
-  return $_[0]->{subject};
-}
-
 sub make_cert {
   my ($self, $args) = @_;
   my $asset = $args->{cert} ? Path::Tiny->new($args->{cert}) : Path::Tiny->tempfile;
 
-  local $UMASK = 0222; # make files with mode 444
-
-  openssl qw( req -new -sha256 -x509 -extensions v3_ca ),
+  local $UMASK = 0222;    # make files with mode 444
+  confess '"subject" is required'
+    unless my $subject = $self->_render_subject($self->subject, $args->{subject});
+  openssl qw(req -new -sha256 -x509 -extensions v3_ca),
     -passin => $self->_passphrase($args->{passphrase}),
-    -days => $args->{days} || DEFAULT_DAYS,
-    -key => $args->{key} || '',
-    -out => $asset->path,
-    -subj => $self->_render_ssl_subject($args->{subject});
+    -days   => $args->{days} || DEFAULT_DAYS,
+    -key    => $args->{key},
+    -out    => $asset->path,
+    -subj   => $subject;
 
   return $asset;
 }
@@ -74,11 +71,11 @@ sub make_crl {
   my ($self, $args) = @_;
   my $asset = $args->{crl} ? Path::Tiny->new($args->{crl}) : Path::Tiny->tempfile;
 
-  local $UMASK = 0122; # make files with mode 644
+  local $UMASK = 0122;    # make files with mode 644
 
-  openssl qw( ca -gencrl ),
+  openssl qw(ca -gencrl),
     -keyfile => $args->{key},
-    -cert => $args->{cert},
+    -cert    => $args->{cert},
     $args->{passphrase} ? (-passin => $self->_passphrase($args->{passphrase})) : (),
     -out => $asset->path;
 
@@ -89,14 +86,16 @@ sub make_csr {
   my ($self, $args) = @_;
   my $asset = $args->{csr} ? Path::Tiny->new($args->{csr}) : Path::Tiny->tempfile;
 
-  local $UMASK = 0277; # make files with mode 400
+  local $UMASK = 0277;    # make files with mode 400
 
-  openssl qw( req -new -sha256 ),
+  confess '"subject" is required'
+    unless my $subject = $self->_render_subject($self->subject, $args->{subject});
+  openssl qw(req -new -sha256),
     $args->{passphrase} ? (-passin => $self->_passphrase($args->{passphrase})) : (),
-    -key => $args->{key},
+    -key  => $args->{key},
     -days => $args->{days} || DEFAULT_DAYS,
-    -out => $asset->path,
-    -subj => $self->_render_ssl_subject($args->{subject});
+    -out  => $asset->path,
+    -subj => $subject;
 
   return $asset;
 }
@@ -108,17 +107,19 @@ sub make_directories {
 
   $home->mkpath;
   -w $home or confess "Cannot write to $home";
-  mkdir $home->child($_) for qw( certs crl newcerts private );
+  mkdir $home->child($_) for qw(certs csr crl newcerts private);
   chmod 0700, $home->child('private') or confess "Could not chmod 0700 'private' in $home";
 
   if ($args->{templates}) {
-    local $UMASK = 0122; # make files with mode 644
+    local $UMASK = 0122;    # make files with mode 644
     $self->render_to_file('crlnumber', $file, {}) unless -e ($file = $home->child('crlnumber'));
     $self->render_to_file('index.txt', $file, {}) unless -e ($file = $home->child('index.txt'));
+    $self->render_to_file('index.txt.attr', $file, {})
+      unless -e ($file = $home->child('index.txt.attr'));
     $self->render_to_file('serial', $file, {}) unless -e ($file = $home->child('serial'));
   }
 
-  return $args->{home}; # TBD, but will be true
+  return $args->{home};     # TBD, but will be true
 }
 
 sub make_key {
@@ -126,15 +127,15 @@ sub make_key {
   my $asset = $args->{key} ? Path::Tiny->new($args->{key}) : Path::Tiny->tempfile;
   my $passphrase;
 
-  local $UMASK = 0277; # make files with mode 400
+  local $UMASK = 0277;      # make files with mode 400
 
   if ($passphrase = $args->{passphrase}) {
     $passphrase = $self->_passphrase($passphrase);
-    Path::Tiny->new($1)->spew({binmode => ':raw'}, $self->_random_passphrase(64)) if $passphrase =~ m!^file:(.+)! and !-e $1;
+    Path::Tiny->new($1)->spew({binmode => ':raw'}, $self->_random_passphrase(64))
+      if $passphrase =~ m!^file:(.+)! and !-e $1;
   }
 
-  openssl 'genrsa',
-    $passphrase ? (-aes256 => -passout => $passphrase) : (),
+  openssl 'genrsa', $passphrase ? (-aes256 => -passout => $passphrase) : (),
     -out => $asset->path,
     $args->{bits} || DEFAULT_BITS;
 
@@ -164,30 +165,36 @@ sub revoke_cert {
 
   local $args->{crl} = $args->{crl} || $home->child('crl.pem');
 
-  openssl qw( ca ),
-    $args->{passphrase} ? (-passin => $self->_passphrase($args->{passphrase})) : (),
+  openssl qw(ca), $args->{passphrase} ? (-passin => $self->_passphrase($args->{passphrase})) : (),
     -revoke => $args->{revoke};
 
-  return $self->make_crl($args); # TBD, but will be true
+  return $self->make_crl($args);    # TBD, but will be true
 }
 
 sub sign_csr {
   my ($self, $args) = @_;
   my $asset = $args->{cert} ? Path::Tiny->new($args->{cert}) : Path::Tiny->tempfile;
 
-  local $UMASK = 0222; # make files with mode 444
+  local $UMASK = 0222;              # make files with mode 444
 
-  openssl qw( ca -batch -notext -md sha256 ),
-    -keyfile => $args->{ca_key},
-    -cert => $args->{ca_cert},
-    -passin => $self->_passphrase($args->{passphrase}),
+  openssl qw(ca -batch -notext -md sha256),
+    -keyfile    => $args->{ca_key},
+    -cert       => $args->{ca_cert},
+    -passin     => $self->_passphrase($args->{passphrase}),
     -extensions => $args->{extensions} || 'usr_cert',
-    -out => $asset->path,
-    -in => $args->{csr};
+    -out        => $asset->path,
+    -in         => $args->{csr};
 
   return $asset;
 }
 
+sub subject {
+  my $self = shift;
+  return $self->{subject} // $ENV{SSLMAKER_SUBJECT} // '' unless @_;
+  $self->{subject} = $self->_render_subject(@_);
+  return $self;
+}
+
 sub with_config {
   my ($self, $cb, $args) = @_;
   my $key = join ':', 'config', map { ($_, $args->{$_} // ''); } @CONFIG_TEMPLATE_KEYS;
@@ -195,7 +202,7 @@ sub with_config {
   local $args->{home} = $self->_home($args);
 
   {
-    local $UMASK = 0177; # read/write for current user
+    local $UMASK = 0177;    # read/write for current user
     $self->{$key} ||= $self->render_to_file('openssl.cnf', $args);
   }
 
@@ -216,12 +223,24 @@ sub _cat {
 
 sub _home {
   my ($self, $args) = @_;
-  return Path::Tiny->new($args->{home}) if exists $args->{home};
+  return Path::Tiny->new($args->{home})              if exists $args->{home};
   return Path::Tiny->new($args->{ca_key})->parent(2) if $args->{ca_key};
-  return Path::Tiny->new($args->{key})->parent(2) if $args->{key};
+  return Path::Tiny->new($args->{key})->parent(2)    if $args->{key};
   confess 'home is required';
 }
 
+sub _parse_subject {
+  my ($self, $val) = @_;
+  return $val if ref $val eq 'HASH';
+
+  # /C=US/ST=Texas/L=Dallas/O=Company/OU=Department/CN=example.com/emailAddress=admin@example.com
+  # Subject: C = US, ST = Texas, L = Dallas, O = Company, OU = Department, CN = superduper
+  my $re = index($val, '/') == 0 ? qr{/([A-Za-z]+)=([^/]*)} : qr{([A-Za-z]+)\s+=\s+([^,]*)};
+  my %subject;
+  $subject{$1} = $2 while $val =~ /$re/g;
+  return \%subject;
+}
+
 sub _passphrase {
   my ($self, $phrase) = @_;
 
@@ -232,27 +251,41 @@ sub _passphrase {
 
 sub _random_passphrase {
   my ($self, $length) = @_;
-  my @chr = ('a'..'z', 'A'..'Z', 0..9);
-  join '', map { $chr[rand @chr] } 1..$length;
+  my @chr = ('a' .. 'z', 'A' .. 'Z', 0 .. 9);
+  join '', map { $chr[rand @chr] } 1 .. $length;
 }
 
-sub _render_ssl_subject {
-  my $self = shift;
-  my $override = shift // '';
-  my %subject = map { split '=', $_, 2 } split '/', $self->subject;
+sub _read_subject_from_cert {
+  my ($self, $cert) = @_;
+  my %subject;
 
-  for my $kv (split '/', $override) {
-    $subject{$1} = $2 if $kv =~ /^([^=]+)=(.*)$/;
+  # Subject: C = US, ST = Texas, L = Dallas, O = Company, OU = Department, CN = superduper
+  return openssl qw(x509 -noout -text -in), $cert => sub {
+    return $self->_parse_subject($1) if $_[1] =~ m!Subject:\s+(.+)!;
+  };
+
+  die qq(Could not read subject from "$cert".);
+}
+
+sub _render_subject {
+  my $self = shift;
+
+  my %subject;
+  for (@_) {
+    next unless $_;
+    my $s = -r ? $self->_read_subject_from_cert($_) : $self->_parse_subject($_);
+    $subject{$_} = $s->{$_} for keys %$s;
   }
 
-  return join '/', '', map { "$_=$subject{$_}" } grep { defined $subject{$_} } qw( C ST L O OU CN emailAddress );
+  return join '/', '',
+    map {"$_=$subject{$_}"} grep { defined $subject{$_} } qw(C ST L O OU CN emailAddress);
 }
 
 # used in script/sslmaker
 sub _render_template {
   my ($self, $name, $stash) = @_;
   my $template = $DATA{$name} // confess "No such template: $name";
-  $template =~ s!<%=\s*([^%]+)\s*%>!{eval $1 // confess $@}!ges; # super cheap template parser
+  $template =~ s!<%=\s*([^%]+)\s*%>!{eval $1 // confess $@}!ges;    # super cheap template parser
   $template;
 }
 
@@ -266,7 +299,7 @@ App::sslmaker - Be your own SSL certificate authority
 
 =head1 VERSION
 
-0.13
+0.14
 
 =head1 DESCRIPTION
 
@@ -278,26 +311,11 @@ the details. "Just work" depends on safe defaults, which will change when
 new and more secure standards come along.
 
 The openssl commands are based on the instructions from
-L<https://jamielinux.com/blog/category/CA/>.
+L<https://jamielinux.com/docs/openssl-certificate-authority/>.
 
 This module is used by the C<sslmaker> command line application, but can also
 act as a standalone toolkit.
 
-From L<https://jamielinux.com/articles/2013/08/act-as-your-own-certificate-authority/>:
-
-  Most websites, such as shopping, banking or email websites, need to let their
-  customers know that the connection is secure. Thus, they need to pay a
-  well-known and internationally trusted CA (eg, VeriSign) to issue an SSL
-  certificate. However, this isn't always necessary. For example, if you're
-  setting up a virtual private network (VPN) or an intranet website, it might
-  make more sense to issue your own certificates.
-
-  Being a CA means dealing with cryptographic pairs of private keys and public
-  certificates. Ideally the cryptographic pairs should be generated in a secure
-  environment, which means a personal laptop or computer that is disconnected
-  from the Internet. It is not recommended to generate any certificates
-  directly on your server.
-
 =head1 DISCLAIMER
 
 This module is based on tips and tricks from online resources, and has been
@@ -306,69 +324,139 @@ any parts involved cannot be held responsible for the security of your
 server, application or other parts that use the files generated by this
 library.
 
-=head1 COMMAND LINE USAGE
-
-  $ sslmaker [options] {action} {arg1,...}
-
-  # Generate root CA key and certificate
-  $ sslmaker --home /path/to/pki/CA root
-
-  # Generate intermediate CA key and certificate
-  $ sslmaker --root-home /path/to/pki/CA --home /path/to/pki/intermediate intermediate
-
-  # Generate client or server key and certificate signing request
-  $ sslmaker --home /path/to/pki/intermediate generate <commonName>
-  $ sslmaker --home /path/to/pki/intermediate generate www.example.com
-  $ sslmaker --home /path/to/pki/intermediate nginx www.example.com
-
-  # Sign a certificate signing request
-  $ sslmaker --home /path/to/pki/intermediate sign www.example.com.csr.pem [outfile]
-
 =head1 SYNOPSIS
 
-  my $sslmaker = App::sslmaker->new(subject => '/C=US/ST=Gotham/L=Gotham/O=Wayne Enterprises/OU=Batcave/CN=batman');
-  my $key = $sslmaker->make_key;
-  my $csr = $sslmaker->make_csr({ key => $key->path });
+  $ sslmaker <action> [options]
 
-  # the content of the csr
-  print $csr->slurp;
+  # 1. Initial CA setup
+  # 1a. The CA admin generates root CA key and certificate
+  $ sslmaker root --subject "/C=US/ST=Texas/L=Dallas/O=Company/OU=Department/CN=superduper"
 
-  # move to a non-temp location
-  $csr->move("/path/to/certs/output.csr.pem");
-  $key->move("/path/to/certs/output.csr.pem");
+  # 1b. The CA admin generates intermediate CA key and certificate
+  # Uses the --subject from root CA by default
+  $ sslmaker intermediate
 
-All methods will throw an exception on error, unless otherwise noted.
+  # 2. Client certificate setup
+  # 2a. The client generates a server key and certificate signing request
+  # Can be done on any other server
+  # Uses the --subject from intermediate CA if available
+  $ sslmaker generate <cn>
+  $ sslmaker generate www.example.com
 
-=head2 RESOURCES
+  # 2b. The client sends the signing request file to the CA admin
+
+  # 3. CA sign and revoke process
+  # 3a. The CA admin signs the certificate request
+  $ sslmaker sign www.example.com.csr.pem
+  $ sslmaker sign www.example.com.csr.pem [outfile]
+
+  # 3b. The CA admin sends back the signed certificate which the client can use
+
+  # 3c. The CA can revoke a certificate
+  $ sslmaker revoke <infile>
+  $ sslmaker revoke /etc/ssl/sslmaker/newcerts/1000.pem
+
+  # 4. Utility commands
+  # 4a. Show certificate details
+  $ sslmaker show <infile>
+  $ sslmaker show /etc/ssl/sslmaker/newcerts/1000.pem
+
+  # 4b. Create dhparam file
+  $ sslmaker dhparam
+  $ sslmaker dhparam /etc/ssl/sslmaker/dhparam.pem 2048
+
+  # 4c. Show the manual for App::sslmaker
+  $ sslmaker man
+
+=head1 ENVIRONMENT VARIABLES
+
+=over 2
+
+=item * SSLMAKER_BITS
+
+Default bits for a generated certificate. Default is 4096.
+
+=item * SSLMAKER_DAYS
+
+Default days before expiring a generated certificate. Default is 365.
+
+=item * SSLMAKER_DEBUG
+
+Setting this to "0" will output less debug information from C<sslmaker>.
+
+=item * SSLMAKER_HOME
+
+Used by the C<sslmaker> script as default home directory. Default is either
+"/etc/pki/sslmaker" or "/etc/ssl/sslmaker".
+
+Directory structure is:
+
+  # generated by "sslmaker root"
+  $SSLMAKER_HOME/root/ca.cert.pem
+  $SSLMAKER_HOME/root/ca.key.pem
+  $SSLMAKER_HOME/root/crlnumber
+  $SSLMAKER_HOME/root/index.txt
+  $SSLMAKER_HOME/root/index.txt.attr
+  $SSLMAKER_HOME/root/passphrase
+  $SSLMAKER_HOME/root/serial
+
+  # generated by "sslmaker intermediate"
+  $SSLMAKER_HOME/certs/ca-chain.cert.pem
+  $SSLMAKER_HOME/certs/ca.cert.pem
+  $SSLMAKER_HOME/certs/ca.csr.pem
+  $SSLMAKER_HOME/private/ca.key.pem
+  $SSLMAKER_HOME/private/passphrase
+  $SSLMAKER_HOME/root/newcerts/1000.pem
+  $SSLMAKER_HOME/crlnumber
+  $SSLMAKER_HOME/index.txt
+  $SSLMAKER_HOME/index.txt.attr
+  $SSLMAKER_HOME/serial
+
+  # generated by "sslmaker sign"
+  $SSLMAKER_HOME/newcerts/1000.pem
+
+  # generated by "sslmaker dhparam"
+  $SSLMAKER_HOME/dhparam.pem
+
+NOTE! After running "sslmaker intermediate", then it is highly suggested to
+move "$SSLMAKER_HOME/root/" to a safe location, such as a memory stick. You can
+revoke any of the child certificates if they are compromised, but if you loose
+the root key, then all is lost.
+
+=item * SSLMAKER_OPENSSL
+
+Default to "openssl". Can be set to a custom path if "openssl" is not in
+C<PATH>.
+
+=item * SSLMAKER_SUBJECT
+
+Used as default subject, unless specified.
+
+=back
+
+=head2 SEE ALSO
 
 =over 4
 
-=item * L<https://jamielinux.com/blog/category/CA>
+=item * L<https://jamielinux.com/docs/openssl-certificate-authority/>
 
 =item * L<https://www.digitalocean.com/community/tutorials/openssl-essentials-working-with-ssl-certificates-private-keys-and-csrs>
 
 =item * L<http://en.wikipedia.org/wiki/Certificate_authority>
 
+=item * L<Easy RSA|https://github.com/OpenVPN/easy-rsa>
+
 =back
 
-=head1 ATTRIBUTES
-
-=head2 subject
-
-  $self = $self->subject('/C=NO/ST=Oslo/L=Oslo/O=Example/OU=Prime/emailAddress=admin@example.com');
-  $str = $self->subject;
-
-Holds the default subject field for the certificate.
-
 =head1 METHODS
 
 =head2 make_cert
 
   $asset = $self->make_cert({
-              key => "/path/to/private/input.key.pem",
+              key        => "/path/to/private/input.key.pem",
               passphrase => "/path/to/passphrase.txt",
-              days => $number_of_days, # default: 365
-              subject => '/C=NO/ST=Oslo', # optional
+              days       => $number_of_days, # default: 365
+              subject    => '/C=NO/ST=Oslo', # optional
             });
 
 This method will generate a SSL certificate using a C<key> generated by
@@ -384,8 +472,8 @@ this method.
 =head2 make_crl
 
   $asset = $self->make_crl({
-              key => "/path/to/private/input.key.pem",
-              cert => "/path/to/cefrt/input.cert.pem",
+              key        => "/path/to/private/input.key.pem",
+              cert       => "/path/to/cefrt/input.cert.pem",
               passphrase => "/path/to/passphrase.txt", # optional
             });
 
@@ -404,10 +492,10 @@ See also L</revoke_cert>.
 =head2 make_csr
 
   $asset = $self->make_csr({
-              key => "/path/to/private/input.key.pem",
+              key        => "/path/to/private/input.key.pem",
               passphrase => "/path/to/passphrase.txt",
-              subject => '/C=NO/ST=Oslo',
-              days => $number_of_days, # default: 365
+              subject    => '/C=NO/ST=Oslo',
+              days       => $number_of_days, # default: 365
             });
 
 This method will generate a SSL certificate signing request using a C<key>
@@ -422,7 +510,7 @@ by passing on C<csr> to this method.
 =head2 make_directories
 
   $self->make_directories({
-    home => "/path/to/pki",
+    home      => "/path/to/pki",
     templates => 1, # default: false
   });
 
@@ -442,7 +530,7 @@ expects. Set C<$emplates> to a true value to generate L<files|/render_to_file>.
 
   $asset = $self->make_key({
               passphrase => "/path/to/passphrase.txt", # optional
-              bits => 8192, # default: 4096
+              bits       => 8192, # default: 4096
             });
 
 This method will generate a SSL key.
@@ -454,14 +542,6 @@ The returned C<$asset> is a L<Path::Tiny> object which holds the generated key.
 It is possible to specify the location of this object by passing on C<key> to
 this method.
 
-From L<https://jamielinux.com/articles/2013/08/act-as-your-own-certificate-authority/>:
-
-  The very first cryptographic pair we generate includes what is known as a
-  root certificate. The root key (ca.key.pem) generated in this step should be
-  kept extremely secure, otherwise an attacker can issue valid certificates for
-  themselves. We'll therefore protect it with AES 256-bit encryption and a
-  strong password just in case it falls into the wrong hands.
-
 =head2 new
 
   $self = App::sslmaker->new(%args);
@@ -497,9 +577,9 @@ See L</TEMPLATES> for list of valid templates.
 
   $self->with_config(
     revoke_cert => {
-      key => "/path/to/private/ca.key.pem",
-      cert => "/path/to/certs/ca.cert.pem",
-      crl => "/path/to/crl.pem",
+      key    => "/path/to/private/ca.key.pem",
+      cert   => "/path/to/certs/ca.cert.pem",
+      crl    => "/path/to/crl.pem",
       revoke => "/path/to/newcerts/1000.pem",
     },
   );
@@ -510,9 +590,9 @@ C<OPENSSL_CONF> or inside L</with_config>.
 =head2 sign_csr
 
   $asset = $self->sign_csr({
-              csr => "/path/to/certs/input.csr.pem",
-              ca_key => "/path/to/private/ca.key.pem",
-              ca_cert => "/path/to/certs/ca.cert.pem",
+              csr        => "/path/to/certs/input.csr.pem",
+              ca_key     => "/path/to/private/ca.key.pem",
+              ca_cert    => "/path/to/certs/ca.cert.pem",
               passphrase => "/path/to/passphrase.txt",
               extensions => "v3_ca", # default: usr_cert
             });
@@ -525,6 +605,16 @@ The returned C<$asset> is a L<Path::Tiny> object which holds the generated
 certificate. It is possible to specify the location of this object by
 passing on C<cert> to this method.
 
+=head2 subject
+
+  $self = $self->subject(@subjects);
+  $self = $self->subject("/C=NO/ST=Oslo/L=Oslo/O=Example/OU=Prime/emailAddress=admin@example.com", ...);
+  $str = $self->subject;
+
+Holds the default subject field for the certificate. Can be set by passing in a
+list of subject strings, hashes or paths to certificate files. The list will
+get merged, soo the last one overrides the one before.
+
 =head2 with_config
 
   $any = $self->with_config($method => \%args);
@@ -534,9 +624,9 @@ file. The C<%stash> in the template will be constructed from the C<%args>,
 which is also passed on to the next C<$method>. Example:
 
   $asset = $self->with_config(make_key => {
-              home => "/path/to/pki",
+              home       => "/path/to/pki",
               passphrase => "/path/to/pki/private/passphrase.txt",
-              bits => 8192,
+              bits       => 8192,
            });
 
 The config file will be removed when C<$self> go out of scope.
@@ -582,7 +672,7 @@ C<%stash>, it will be used as the start number, which defaults to 1000.
 
 =head2 Code
 
-Copyright (C) 2014, Jan Henning Thorsen
+Copyright (C) Jan Henning Thorsen
 
 The code is free software, you can redistribute it and/or modify it under the
 terms of the Artistic License version 2.0.
@@ -605,6 +695,7 @@ __DATA__
 @@ crlnumber
 <%= $stash->{n} || 1000 %>
 @@ index.txt
+@@ index.txt.attr
 @@ serial
 <%= $stash->{n} || 1000 %>
 @@ nginx.config
@@ -647,7 +738,6 @@ crlnumber = $dir/crlnumber
 crl = <%= $stash->{crl} || '$dir/crl.pem' %>
 private_key = <%= $stash->{key} || '$dir/private/ca.key.pem' %>
 RANDFILE = $dir/private/.rand
-x509_extensions = usr_cert
 crl_extensions = crl_ext
 name_opt = ca_default
 cert_opt = ca_default
@@ -656,6 +746,18 @@ default_crl_days = <%= $stash->{crl_days} || 30 %>
 default_md = <%= $stash->{default_md} || 'sha256' %>
 preserve = no
 policy = policy_anything
+x509_extensions = basic_exts
+
+[ basic_exts ]
+basicConstraints = CA:FALSE
+subjectKeyIdentifier = hash
+authorityKeyIdentifier = keyid,issuer:always
+
+[ custom_req_extensions ]
+subjectKeyIdentifier=hash
+authorityKeyIdentifier=keyid:always,issuer:always
+basicConstraints = CA:true
+keyUsage = cRLSign, keyCertSign
 
 [ policy_anything ]
 countryName = optional
@@ -663,7 +765,7 @@ stateOrProvinceName = optional
 localityName = optional
 organizationName = optional
 organizationalUnitName = optional
-commonName = supplied
+commonName = optional
 emailAddress = optional
 
 [ req ]
@@ -672,8 +774,8 @@ default_md = sha1
 default_keyfile = privkey.pem
 distinguished_name = req_distinguished_name
 attributes = req_attributes
-x509_extensions = v3_ca
 string_mask = utf8only
+x509_extensions = custom_req_extensions
 
 [ req_distinguished_name ]
 
@@ -693,7 +795,7 @@ basicConstraints = critical,CA:true
 keyUsage = cRLSign, keyCertSign
 
 [ crl_ext ]
-authorityKeyIdentifier=keyid:always
+authorityKeyIdentifier=keyid:always,issuer:always
 
 [ proxy_cert_ext ]
 basicConstraints = CA:FALSE
